; dimmer
; remote control data
;			type	start pulse, ms	pulse, ms	quantity	period, ms	pause, ms
; Technics	1		3,8 / 1,8		0,8			st + 61		133			74
; Humax		2		9 / 4			1			st + 42		108			47
; NTV		2		9 / 4			1			st + 43		108			47
; Tascam	2		9 / 4			1			st + 42		108			42
; Pioneer	1		8 / 4			1			st + 49		90			25
; Computer	2		9 / 4			1			st + 64		109			23
; JVC mc	3		8 / 4			1			st + 27		60			18
; JVC tv	3		8 / 4			1			st + 20		60			25
; Strong	2		8 / 4			1			st + 49		107			42

; start pulse: 3,8 ms to 10 ms
; data end if > 4,0 ms
; 1,3 ms < bit 0 < 3,5 ms
; 0,5 ms < bit 1 < 1,3 ms
; 15 ms < pause < 100 ms
;*****************
.include "tn13def.inc"
;*****************
; pcb version
.equ PCB	= 2
; constants
; in-out port constants
.equ OUTPORT	= PORTB
.equ OUTPORT_DIRECTION	= DDRB
.equ INPORT		= PINB
.equ RESETPIN	= PORTB5
.IF PCB == 1
.equ FIRE		= PORTB4
.equ PHASE_INPUT	= PORTB2
.equ KEY		= PORTB0
.equ IR			= PORTB3
.equ LED		= PORTB1
.ELSE
.IF PCB == 2
.equ FIRE		= PORTB4
.equ PHASE_INPUT	= PORTB3
.equ KEY		= PORTB2
.equ IR			= PORTB0
.equ LED		= PORTB1
.ENDIF
.ENDIF
; time constants
.equ QUARTZ		= 4800; quartz frequency, kHz
.equ PRESCALER	= 256
.equ HALFPERIOD_MIN = (7 * QUARTZ / PRESCALER); 7 ms
.equ HALFPERIOD_MAX = (12 * QUARTZ / PRESCALER); 12 ms
.equ TRIAC_FIRE_TIME = (150 * QUARTZ / PRESCALER + 500) / 1000; 150 mks
.equ KEY_PRESS_MIN	= 5; n * 10 ms, protect time, 1...7
.equ KEY_PRESS_LONG	= 70; n * 10 ms, long press and full light switch on
.equ IR_PRESS_MIN	= 1; n * 10 ms, 1...7
.equ IR_PRESS_LONG	= 100; n * 10 ms
; ir constants
.equ MIN_COMMAND_LENGTH	= 8
.equ NUMBER_OF_COMMAND	= 7; 1...7
.equ IR_MAX_PULSE	= (3500 * QUARTZ / PRESCALER + 500) / 1000; 3.5 ms
.equ IR_MIN_PULSE	= (500 * QUARTZ / PRESCALER + 500) / 1000; 0,5 ms
.equ IR_END_CMD		= (10000 * QUARTZ / PRESCALER + 500) / 1000; 10.0 ms
.equ IR_BIT_COMPARE	= (250 * QUARTZ / PRESCALER + 500) / 1000; 250 mks
.equ IR_EDGE_COMPARE	= (250 * QUARTZ / PRESCALER + 500) / 1000; 250 mks
.equ IR_MAX_PULSE_V2	= (10000 * QUARTZ / PRESCALER + 500) / 1000; 10.0 ms
.equ IR_PAUSE		= 120 * QUARTZ / PRESCALER; 120 ms
; brightness constants
.equ BR_LP_STEP	= 96; 1...255
.equ BR_SP_STEP_START	= 32; 1...128
.equ BR_SP_STEP	= 512; 1...10000
.equ BR_ON_HEAT	= 6; 4...10
.equ BR_MIN		= 30; 0...127
.equ BR_MIN_DEF	= 3; 2...32
.equ BR_TIMER_OFF	= 55; 12 hours
; flags
.equ F_Reg		= 7; regulation
.equ F_Dir		= 6; 0 - decrement brightness, 1 - increment
.equ F_IR_Cmd	= 5; 1 - ir command have received
.equ F_IR_Pulse	= 4; 1 - ir command start pulse have detected
.equ F_Key_SPress	= 3; 1 - single press
.equ F_IR_SPress	= 2; 1 - single press
.equ F_Key_LPress	= 1; 1 - long press
.equ F_IR_LPress	= 0; 1 - long press
; led flash
.equ LED_ON		= 1; n * 10ms, device on
.equ LED_OFF	= 0; n * 10ms, device off
.equ LED_WR		= 1; n * 10ms, ir control programming mode
.equ LED_COM	= 7; n * 10ms, ir control command receive
; variables
.def zero		= r8
.def old_pin	= r9
.def eeprom_data	= r10
.def br_memory	= r11
.def half_period	= r12
.def pulse_time	= r13
.def st_store	= r14
.def old_timer	= r15

.def temp		= r16
.def flags		= r17
.def write_mode	= r18
.def led_counter	= r19
.def bit_counter	= r20
.def brightness_l	= r21
.def brightness_h	= r22
.def key_rep_counter	= r23
.def ir_rep_counter	= r24
.def timer_ext_3	= r25
.def timer_ext_1	= r30
.def timer_ext_2	= r31
;.def 
; r0 ... r7 ir command
; YL - low byte of old timer
; YH - high byte of old timer
;*****************
; memory allocate
.dseg
TABLE:	.BYTE NUMBER_OF_COMMAND * 8
;-----------------
; flash memory allocate
; first 8 bytes * NUMBER_OF_COMMAND flash memory - for store commands
.eseg
IRCOMMAND:	.BYTE	NUMBER_OF_COMMAND * 8
BRIGHTNESS_MEM:	.BYTE	1
;*****************
.cseg
.org 0
	rjmp reset
	reti; EXT_INT0, IRQ0 Handler
	rjmp PHASE_INT; phase control, PCINT0 Handler
	rjmp TIM0_OVF ; width enhanced, Timer0 Overflow Handler
	reti;rjmp EE_RDY ; EEPROM Ready Handler
	reti;rjmp ANA_COMP ; Analog Comparator Handler
	rjmp TIM0_COMPA ; Triac FIRE! Timer0 CompareA Handler
	rjmp TIM0_COMPB ; Triac OFF! Timer0 CompareB Handler
	reti;rjmp WATCHDOG ; Watchdog Interrupt Handler
	reti;rjmp ADC_INT ; ADC Conversion Handler
;****************
; program info
.db "ir remote dimmer for incandescent lamp"
.db "(c) 2006 V*D*V v1.0"
;****************
; initialisation
reset:
	cli
	ldi	temp, low(RAMEND)
	out	spl, temp
;----------------
;	wdr
; set modes
;	ldi	temp, ((0 << WDE) | (0 << WDTIE) | (1 << WDP3) | (0 << WDP2) | (0 << WDP1) | (1 << WDP0))
;	out	WDTCR, temp
; watch dog timer initialization
	wdr
	ldi	temp, (1 << WDCE) | (1 << WDE)
	out	WDTCR, temp
	ldi	temp, (1 << WDE) | (0 << WDCE) | (0 << WDTIE) | (0 << WDP3) | (1 << WDP2) | (1 << WDP1) | (1 << WDP0)
	out	WDTCR, temp
;----------------
	sbi	ACSR, ACD; comparator disable
	clr	zero
	out	MCUSR, zero
	out	GTCCR, zero
	out	CLKPR, zero
	out	TCCR0A, zero
;	ldi	temp, (0 << PUD) | (0 << SE) | (0 << SM1) | (0 << SM0) | (1 << ISC01) | (0 << ISC00)
;	out	MCUCR, temp
	out	MCUCR, zero
;----------------
; set default port states, PUD in MCUSR must be 0
	ldi	temp, (1 << FIRE) | (1 << KEY) | (1 << RESETPIN) | (1 << LED)
	out OUTPORT, temp
; port directions
; DDR 1 - output, PORT must be 0
; DDR 0 - input, if PORT=0 - tristate
	ldi	temp, (1 << FIRE) | (1 << LED)
	out	OUTPORT_DIRECTION, temp
;----------------
; set timer settings for measuring
.IF PRESCALER == 256
	ldi	temp, 0b00000100; timer prescaler clk/256
.ELSE
.IF PRESCALER == 64
	ldi	temp, 0b00000011; timer prescaler clk/64
.ELSE
.IF PRESCALER == 8
	ldi	temp, 0b00000010; timer prescaler clk/8
.ELSE
.IF PRESCALER == 1
	ldi	temp, 0b00000001; timer prescaler clk/1
.ELSE
.IF PRESCALER == 0
	ldi	temp, 0b00000000; timer prescaler no clk
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
	out	TCCR0B, temp
;---------------
; read ir commands from eeprom memory
	clr		XH
	ldi		XL, NUMBER_OF_COMMAND * 8 + TABLE; start table of command
a_read_repeat:
	mov		temp, XL
	subi	temp, TABLE + 1
	rcall	sr_eeprom_read
	st		-X, temp
	cpi		XL, TABLE
	brne	a_read_repeat
;---------------
; set timer
	clr	timer_ext_1
	clr	timer_ext_2
	clr	timer_ext_3
;---------------
; data initialisation
	clr		write_mode
	clr		ir_rep_counter
	clr		key_rep_counter
	rcall	ir_exit_clear
	ldi		temp, HALFPERIOD_MIN - TRIAC_FIRE_TIME; - BR_MIN_DEF
	mov		half_period, temp
; set brightness
	clr		brightness_l
	ser		brightness_h
	ldi		temp, BRIGHTNESS_MEM
	rcall	sr_eeprom_read
	mov		br_memory, temp
; flags
	clr	flags
;------------------
; external interrupt configure for phase control
	ldi	temp, (1 << PHASE_INPUT)
	out	PCMSK, temp; phase control
	ldi	temp, (0 << INT0) | (1 << PCIE)
	out	GIMSK, temp
;-----------------
; timer init
	out	TCNT0, zero
	out	OCR0B, zero
	ser	temp
	out	OCR0A, temp
	out	TIFR0, temp
;
	in	YL, TCNT0
	mov	YH, timer_ext_1
; timer interrupt configure
	mov	old_pin, temp
	ldi	temp, (1 << TOIE0) | (1 << OCIE0A) | (1 << OCIE0B)
	out	TIMSK0, temp
;----------------
; led state
	ldi	led_counter, LED_ON
;----------------
; main
	sei
main_loop:
	wdr
	rcall	IR_READ
	rjmp	main_loop
;****************
; temp - address, temp - data
sr_eeprom_read:
EEPROM_read:
; Wait for completion of previous write
	sbic	EECR, EEPE
	rjmp	EEPROM_read
; Set up address (temp) in address register
	out		EEARL, temp
; Start eeprom read by writing EERE
	sbi		EECR, EERE
; Read data from data register
	in		temp, EEDR
	ret
;****************
; timer interrupt
TIM0_OVF:
	in		st_store, SREG
	adiw	timer_ext_1, 1
;	adc		timer_ext_2, 0
	adc		timer_ext_3, zero
	out		SREG, st_store
	reti
;****************
; Triac FIRE! (comparator 1 interrupt)
TIM0_COMPA:
;	in		st_store, SREG
;	cp		brightness_h, half_period
;	brcc	a_fire_exit
	cbi		OUTPORT, FIRE
a_fire_exit:
;	out		SREG, st_store
	reti
;****************
; Triac OFF! (comparator 2 interrupt)
TIM0_COMPB:
	sbi		OUTPORT, FIRE
	reti
;****************
; phase control (pcint0 interrupt)
PHASE_INT:
	sbi		OUTPORT, FIRE
	in		st_store, SREG
	push	temp
a_phase_cntrl:
	ldi		temp, (1 << PSR10)
	out		GTCCR, temp
	in		temp, TCNT0
	add		temp, brightness_h
	out		OCR0A, temp
	subi	temp, -TRIAC_FIRE_TIME
	out		OCR0B, temp
;	ldi		temp, (1 << OCF0A)
;	out		TIFR0, temp
;----------------
; correction of phase
	in		temp, TCNT0
	sub		temp, old_timer
	in		old_timer, TCNT0
; must be check to valid value
; 15 ms < period < 22 ms
	cpi		temp, HALFPERIOD_MIN
	brcs	a_msr_not_correct
	cpi		temp, HALFPERIOD_MAX
	brcc	a_msr_not_correct
;
	subi	temp, TRIAC_FIRE_TIME + BR_MIN_DEF
	sbis	INPORT, PHASE_INPUT
	mov		half_period, temp
a_msr_not_correct:
;****************
; key pressed
	rcall	KEY_LOGIC
	mov		temp, key_rep_counter
	or		temp, ir_rep_counter
	brne	a_clear_timer
; light off timer control
	cpi		timer_ext_3, BR_TIMER_OFF
	brcs	a_logic
	cpi		brightness_h, 255
	breq	a_clear_timer
	ldi		flags, (0 << F_Dir) | (1 << F_Key_SPress)
a_clear_timer:
	clr		timer_ext_2
	clr		timer_ext_3
;
a_logic:
	ldi		temp, LED_WR
	tst		write_mode
	brtc	a_wr_cmd
	ldi		temp, LED_COM
a_wr_cmd:
	brne	a_led_flash
	rcall	IR_LOGIC
	rcall	LONG_PRESS
	rcall	SHORT_PRESS
; led logic
	ldi		temp, LED_OFF
	cpi		brightness_h, 255
	brne	a_led_cmd
	ldi		temp, LED_ON
a_led_cmd:
	sbrc	flags, F_IR_Cmd
	ldi		temp, LED_COM
; led flash
	cpi		temp, LED_OFF
	breq	a_led_off
a_led_flash:
	dec		led_counter
	brne	a_led_no_change
	mov		led_counter, temp
	sbis	OUTPORT, LED
	rjmp	a_led_off
	cbi		OUTPORT, LED
a_led_no_change:
	pop		temp
	out		SREG, st_store
	reti
;----------------
a_led_off:
	sbi		OUTPORT, LED
	rjmp	a_led_no_change
;****************
; key logic
KEY_LOGIC:
	sbic	INPORT, KEY
	rjmp	a_kl_release
	cpi		key_rep_counter, 255
	adc		key_rep_counter, zero
	cpi		key_rep_counter, KEY_PRESS_LONG
	brcs	a_kl_exit
; for programming
	tst		ir_rep_counter
	brne	a_kl_set_lp
	sbrs	flags, F_Key_LPress
	brts	a_kl_wr_mode
a_kl_set_lp:
	ori		flags, (1 << F_Key_LPress)
a_kl_exit:
	ret
;------------
a_kl_release:
	mov		temp, key_rep_counter
	clr		key_rep_counter
	tst		write_mode
	breq	a_kl_test_press
	clr		write_mode
	andi	flags, ~((1 << F_Key_SPress) | (1 << F_Key_LPress))
	ret
;------------
a_kl_test_press:
	cpi		temp, KEY_PRESS_MIN
	brcs	a_kl_exit
	cpi		temp, KEY_PRESS_LONG
	brcc	a_kl_lp_rel
	ori		flags, (1 << F_Key_SPress)
	ret
;------------
a_kl_lp_rel:
	andi	flags, ~((1 << F_Key_SPress) | (1 << F_Key_LPress))
;
	mov		temp, half_period
	subi	temp, BR_MIN
	cp		temp, brightness_h
	brcs	a_kl_on_min
;
a_lp_rel:
	ldi		temp, 1 << F_Dir
	eor		flags, temp
;****************
; subroutine store brightness
sr_store_brightness:
	ldi		temp, BRIGHTNESS_MEM
	mov		eeprom_data, brightness_h
;******************
; write to eeprom
; temp - address, eeprom_data - data
sr_eeprom_write:
; Wait for completion of previous write
	sbic	EECR, EEPE
	rjmp	sr_eeprom_write
	out		EEARL, temp
	out		EEDR, eeprom_data
	cbi		EECR, EEPM0
	cbi		EECR, EEPM1
;	in		eeprom_data, SREG
;	cli
	sbi		EECR, EEMPE
	sbi		EECR, EEPE
;	out		SREG, eeprom_data
	ret
;-----------
a_kl_on_min:
	ori		flags, (1 << F_Key_SPress) | (1 << F_Dir) | (1 << F_Reg)
	mov		br_memory, temp
	ret
;-----------
a_kl_wr_mode:
	tst		write_mode
	brne	a_kl_wr_mode_exit
	ldi		write_mode, 1;NUMBER_OF_COMMAND + 1
a_kl_wr_mode_exit:
	ret
;****************
; ir logic
IR_LOGIC:
	sbrs	flags, F_IR_Cmd
	rjmp	a_ir_release
	cpi		ir_rep_counter, 255
	adc		ir_rep_counter, zero
;
	cpi		ir_rep_counter, IR_PRESS_LONG
	brcs	a_ir_exit
	ori		flags, (1 << F_IR_LPress)
a_ir_exit:
	ret
;------------
a_ir_release:
	mov		temp, ir_rep_counter
	clr		ir_rep_counter
	cpi		temp, IR_PRESS_MIN
	brcs	a_ir_exit
	cpi		temp, IR_PRESS_LONG
	brcc	a_ir_lp_rel
	ori		flags, (1 << F_IR_SPress)
	ret
;------------
a_ir_lp_rel:
	andi	flags, ~((1 << F_IR_SPress) | (1 << F_IR_LPress))
;
	mov		temp, half_period
	subi	temp, BR_MIN
	cp		temp, brightness_h
	brcc	a_lp_rel
;a_ir_on_min:
	ori		flags, (1 << F_IR_SPress) | (1 << F_Dir) | (1 << F_Reg)
	mov		br_memory, temp
	ret
;****************
; long press
LONG_PRESS:
	mov		temp, flags
	andi	temp, (1 << F_Key_LPress) | (1 << F_IR_LPress)
	breq	a_lp_ret
	sbrc	flags, F_Dir
	rjmp	a_lp_dec
;-------------
; long press increment brightness
	cpi		brightness_h, 255
	brcs	a_lp_inc
	mov		brightness_h, half_period
a_lp_inc:
	tst		brightness_h
	breq	a_lp_change_dir
;
	subi	brightness_l, BR_SP_STEP_START
	sbc		brightness_h, zero
	mov		temp, half_period
	subi	temp, BR_ON_HEAT
	cp		brightness_h, temp
	brcc	a_lp_exit
;
	subi	brightness_l, (BR_LP_STEP - BR_SP_STEP_START)
	sbc		brightness_h, zero
a_lp_exit:
	mov		br_memory, brightness_h
	ret
;
a_lp_change_dir:
	ldi		temp, (1 << F_Dir)
	eor		flags, temp
a_lp_ret:
	ret
;--------------
a_lp_dec:
;
	ldi		temp, BR_LP_STEP
	add		brightness_l, temp
	adc		brightness_h, zero
	mov		br_memory, brightness_h
;
	mov		temp, half_period
	subi	temp, BR_MIN
	cp		temp, brightness_h
	brcs	a_lp_change_dir
	ret
;****************
; short press
SHORT_PRESS:
	sbrc	flags, F_Reg
	rjmp	a_sp_reg
	mov		temp, flags
	andi	temp, (1 << F_Key_SPress | 1 << F_IR_SPress)
	breq	a_sp_exit
	ori		flags, (1 << F_Reg)
	cpi		brightness_h, 255
	brne	a_sp_off
	mov		brightness_h, half_period
	mov		temp, half_period
	subi	temp, BR_MIN
	cp		temp, br_memory
	brcc	a_sp_state
	clr		br_memory
a_sp_state:
	ori		flags, (1 << F_Dir)
a_sp_clr:
	andi	flags, ~(1 << F_Key_SPress | 1 << F_IR_SPress)
	ret
;--------------
a_sp_off:
	andi	flags, ~(1 << F_Dir)
	mov		br_memory, brightness_h
	rjmp	a_sp_clr
;--------------
a_sp_reg:
	sbrc	flags, F_Dir
	rjmp	a_sp_on
;-------------
; short press off
	ldi		temp, low(BR_SP_STEP)
	add		brightness_l, temp
	ldi		temp, high(BR_SP_STEP)
	adc		brightness_h, temp
;
	cp		half_period, brightness_h
	brcc	a_sp_exit
	ser		brightness_h
	clr		brightness_l
	andi	flags, ~(1 << F_Reg)
a_sp_exit:
	andi	flags, ~(1 << F_Key_SPress | 1 << F_IR_SPress)
	ret
;--------------
a_sp_on:
;
	subi	brightness_l, BR_SP_STEP_START
	sbc		brightness_h, zero
	mov		temp, half_period
	subi	temp, BR_ON_HEAT
	cp		brightness_h, temp
	brcc	a_sp_on_test
;
	subi	brightness_l, low(BR_SP_STEP - BR_SP_STEP_START)
	sbci	brightness_h, high(BR_SP_STEP - BR_SP_STEP_START)
	brcs	a_sp_on_set
;
a_sp_on_test:
	cp		br_memory, brightness_h
	brcs	a_sp_exit
a_sp_on_set:
	mov		brightness_h, br_memory
	clr		brightness_l
	andi	flags, ~(1 << F_Reg | 1 << F_Key_SPress | 1 << F_IR_SPress)
	ret
;****************
; ir read. test pin and commands decoding
; inc repeat counter if receive valid command
; if T-flag set - enable save mode (repeat sequence have detected)
IR_READ:
	mov		temp, old_pin
	in		old_pin, INPORT
	sbrs	flags, F_IR_Pulse
	rjmp	ir_edge_chk
	sbrs	old_pin, IR
	rjmp	ir_no_edge
	sbrc	temp, IR
	rjmp	ir_no_edge; no rising edge
;
	in		XL, TCNT0
	mov		XH, timer_ext_1
	sub		XL, YL
	sbc		XH, YH		; dT
	in		YL, TCNT0	; fix timer counter
	mov		YH, timer_ext_1
	or		XH, XH
	brne	CLEAR_COMMAND
	cpi		XL, IR_MAX_PULSE_V2; 10 ms; IR_MAX_PULSE; 3.5 ms
	brcc	CLEAR_COMMAND
;
	cpi		XL, IR_MIN_PULSE; 0,5 ms
	brcs	ir_exit_error
;
	mov		temp, pulse_time
	mov		pulse_time, XL
	sub		XL, temp
	brcc	ir_mod
	neg		XL
ir_mod:
	cpi	XL, IR_BIT_COMPARE; 0,25 ms
	rol	r0
	rol	r1
	rol	r2
	rol	r3
	rol	r4
	rol	r5
	rol	r6
	rol	r7
	inc	bit_counter
	ret
;----------------
ir_edge_chk:
	eor		temp, old_pin
	andi	temp, (1 << IR)
	breq	ir_no_edge
;
	in		XL, TCNT0
	mov		XH, timer_ext_1
	sub		XL, YL
	sbc		XH, YH		; dT
	in		YL, TCNT0	; fix timer counter
	mov		YH, timer_ext_1
	or		XH, XH
	brne	CLEAR_COMMAND
	tst	bit_counter
	brne	ir_decode
	cpi		XL, IR_MAX_PULSE; 3.5 ms
	brcc	ir_startpulse
;
	cpi		XL, IR_MIN_PULSE; 0,5 ms
	brcs	ir_startpulse;ir_exit_error
;
ir_decode:
	mov		temp, pulse_time
	mov		pulse_time, XL
	sub		XL, temp
	brcc	ir_modul
	neg		XL
ir_modul:
	cpi	XL, IR_EDGE_COMPARE; 0,25 ms
	rol	r0
	rol	r1
	rol	r2
	rol	r3
	rol	r4
	rol	r5
	rol	r6
	rol	r7
	inc	bit_counter
	ret
;----------------
ir_startpulse:
	ori		flags, (1 << F_IR_Pulse)
	ret
;****************
ir_exit_clear:
	andi	flags, ~((1 << F_IR_Cmd) | (1 << F_IR_Pulse))
ir_exit_error:
	clt
;****************
; clear bit sequence and counter
CLEAR_COMMAND:
	clr	r0
	clr	r1
	clr	r2
	clr	r3
	clr	r4
	clr	r5
	clr	r6
	clr	r7
	clr	bit_counter
	clr	pulse_time
ir_exit_ret:
	ret
;***************
ir_no_edge:
	in		XL, TCNT0
	mov		XH, timer_ext_1
	sub		XL, YL
	sbc		XH, YH; dT
	mov		temp, XH
	cpi		XL, low(IR_PAUSE); 120 ms
	sbci	temp, high(IR_PAUSE); 120 ms
	brcc	ir_exit_clear
	tst		XH
	brne	ir_test
	subi	XL, IR_END_CMD; 10.0 ms
	brcs	ir_exit_ret; sequence not complete
;
ir_test:
	tst		bit_counter
	breq 	ir_exit_ret
	sbrs	old_pin, IR
	rjmp	ir_exit_error
	brtc	ir_test_cmd
	sbrc	flags, F_IR_Cmd
	rjmp	ir_repeat_receive
	rjmp	CLEAR_COMMAND
ir_test_cmd:
	cpi		bit_counter, MIN_COMMAND_LENGTH
	brcs	CLEAR_COMMAND;ir_test_exit
	tst		write_mode
	breq	a_ir_cmd_check
;-----------
ir_store_command:
;	clr		XH
	mov		temp, write_mode
	lsl		temp
	lsl		temp
	lsl		temp; *8
	ldi		XL, TABLE - 8; start table of command
	add 	XL, temp
	st		X+, r0
	st		X+, r1
	st		X+, r2
	st		X+, r3
	st		X+, r4
	st		X+, r5
	st		X+, r6
	st		X+, r7
a_st_repeat:
	ld		eeprom_data, -X
	mov		temp, XL
	subi	temp, TABLE
;	rcall	sr_eeprom_write
; write to eeprom. no call subroutine for stack save
; temp - address, eeprom_data - data
wait_eeprom_write:
; Wait for completion of previous write
	sbic	EECR, EEPE
	rjmp	wait_eeprom_write
	out		EEARL, temp
	out		EEDR, eeprom_data
	cbi		EECR, EEPM0
	cbi		EECR, EEPM1
;	in		eeprom_data, SREG
	cli
	sbi		EECR, EEMPE
	sbi		EECR, EEPE
;	out		SREG, eeprom_data
	sei
;
	andi	temp, 7
	brne	a_st_repeat
	inc		write_mode
	cpi		write_mode, NUMBER_OF_COMMAND + 1
	brcs	a_st_cmd_protect
;a_ir_wait_next_command:
	ldi		write_mode, 1
a_st_cmd_protect:
	set
;	ori		flags, 1 << F_IR_Cmd
	ret
;--------------
a_ir_cmd_check:
	ldi		bit_counter, NUMBER_OF_COMMAND * 8 - 8
ir_check_repeat:
;	clr		XH
	ldi		XL, TABLE; start table of command
	add 	XL, bit_counter
	ld		temp, X+
	cp		r0, temp
	brne 	ir_repeat_1
	ld		temp, X+
	cp		r1, temp
	brne 	ir_repeat_1
	ld		temp, X+
	cp		r2, temp
	brne 	ir_repeat_1
	ld		temp, X+
	cp		r3, temp
	brne 	ir_repeat_1
	ld		temp, X+
	cp		r4, temp
	brne 	ir_repeat_1
	ld		temp, X+
	cp		r5, temp
	brne 	ir_repeat_1
	ld		temp, X+
	cp		r6, temp
	brne 	ir_repeat_1
	ld		temp, X+
	cp		r7, temp
	brne 	ir_repeat_1
ir_repeat_receive:
	ori		flags, 1 << F_IR_Cmd
ir_test_exit:
	set
	rjmp	CLEAR_COMMAND
;------------
ir_repeat_1:
	subi	bit_counter, 8
	brpl 	ir_check_repeat
	rjmp	ir_test_exit
